	#####################################
	# Begining of Drybean data analysis #
      # To reproduce the Table 9 and 10, please implement everything 
      # until the end of Dry bean data analysis.
	#####################################

	## Load functions ##
rm(list=ls())

library(mvtnorm)
library(mclust)
source('dis_function.R')

	## Load Data ##

dat <- read.csv('Dry_Bean_Dataset.csv',header = TRUE)
class <- dat[,17] ## class memberships
cls <- names(table(class))
ncls <- length(cls)
for (i in 1:ncls){
  id <- which(class == cls[i])
  class[id] <- i
}
class <- as.numeric(class)

dat <- dat[,-17]
N <- dim(dat)[1]

	## Drop two variables that are severely left-skewed ##
# hist(dat[,10])
# hist(dat[,16])
dat <- dat[,-c(10,16)]
dat[,1] <- log(dat[,1])
dat[,2] <- log(dat[,2])
dat[,3] <- log(dat[,3])
dat[,4] <- log(dat[,4])
dat[,7] <- log(dat[,7])
dat <- scale(dat, center = TRUE, scale = TRUE)
	save.image(“R_Result_DryBean.RData”)

	## Case I : No disagreement problem ##

prop <- 1/ncls # proportion of testing data
N.st <- floor(N*prop)
N.tr <- N - N.st
niter <- 100
Real.pred.mat1 <- matrix(0,niter,2)
Power.1 <- rep(0,niter)
for (iter in 1:niter){
  set.seed(iter)
  test.id <- sample(1:N, N.st, replace = FALSE)
  train.dat <- dat[-test.id,]
  test.dat <- dat[test.id,]
  train.cls <- class[-test.id]
  test.cls <- class[test.id]

  train.mod <- Mclust(train.dat, 7) # Because we know that there are seven clusters
  test.pred <- predict(train.mod, test.dat)$classification
  Real.pred.mat1[iter,1] <- Rand.index(test.pred, test.cls) # Testing accuracy under conventional method
  test.pred.new <- Mclust.machine(train.dat, test.dat, mod=train.mod, ncls = 7, nboot = 100, alpha = 0.05, method='norm')$pred
  Real.pred.mat1[iter,2] <- Rand.index(test.pred.new, test.cls) # Testing accuracy under proposed method

  dis <- which(test.cls == 2) ## individuals who are disagreed in the testing data
  dis.diag <- which(test.pred.new == 99) ## individuals who are diagonosed as disagreed in the testing data
  Power.1[iter] <- sum(dis.diag %in% dis) / length(dis.diag) ## Proportion of testing data who are correctly diagonosed: Power 
}
colnames(Real.pred.mat1) <- c("Mclust","Two-stage")
	save.image(“R_Result_DryBean.RData”)

	## Case II : Disagreement problem on the smallest class ##
niter <- 100
Real.pred.mat2 <- matrix(0,niter,2)
Power.2 <- rep(0,niter)
for (iter in 1:niter){
  set.seed(iter)
  dis.id <- which(class == 2) # The second class was disagreed
  tmp <- sample((1:N)[-dis.id], N.st-length(dis.id), replace = FALSE)
  test.id <- c(tmp,dis.id);rm(tmp) # test data contains class II + more individuals
  train.dat <- dat[-test.id,]
  test.dat <- dat[test.id,]
  train.cls <- class[-test.id]
  test.cls <- class[test.id]
  dis.id <- which(test.cls == 2) # Disagreed individuals
  test.cls[dis.id] <- 99 # Disagreed memberships 
  prop <- length(dis.id) / length(test.cls) # 522/3611

  train.mod <- Mclust(train.dat, 6) # Because we know that there are now six clusters only
  test.pred <- predict(train.mod, test.dat)$classification
  Real.pred.mat2[iter,1] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) # Testing accuracy under conventional method

  test.pred.new <- Mclust.machine(train.dat, test.dat, mod=train.mod, ncls = 6, nboot = 100, alpha = 0.05, method='norm')$pred
  Real.pred.mat2[iter,2] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) + sum(test.pred.new[dis.id] == 99)/length(dis.id)*prop

  dis.diag <- which(test.pred.new == 99) ## individuals who are diagonosed as disagreed in the testing data
  Power.2[iter] <- sum(dis.diag %in% dis.id) / length(dis.diag) ## Proportion of testing data who are correctly diagonosed: Power 
}
colnames(Real.pred.mat2) <- c("Mclust","Two-stage")
	save.image(“R_Result_DryBean.RData”)

	## Case III : Disagreement problem on the smallest class ##
niter <- 100
N.tr <- 10000
N.st <- N - 10000
test.id <- which(class %in% c(1,2)) # The second class was 
Real.pred.mat3 <- matrix(0,niter,2)
Power.3 <- rep(0,niter)
for (iter in 1:niter){
  set.seed(iter)
  dis.id.tmp <- which(class == 2) # The second class was disagreed
  dis.id <- dis.id.tmp[sample(1:522,100,replace = FALSE)]
  tmp <- sample((1:N)[-dis.id.tmp], N.st-length(dis.id), replace = FALSE)
  test.id <- c(tmp,dis.id);rm(tmp) # test data contains class II + more individuals
  train.dat <- dat[-test.id,]
  test.dat <- dat[test.id,]
  train.cls <- class[-test.id]
  test.cls <- class[test.id]
  dis.id <- which(test.cls == 2) # Disagreed individuals
  test.cls[dis.id] <- 99 # Disagreed memberships 
  prop <- length(dis.id) / length(test.cls) # 100/3611

  train.mod <- Mclust(train.dat, 6) # Because we know that there are now five clusters only
  test.pred <- predict(train.mod, test.dat)$classification
  Real.pred.mat3[iter,1] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) # Testing accuracy under conventional method

  test.pred.new <- Mclust.machine(train.dat, test.dat, mod=train.mod, ncls = 6, nboot = 100, alpha = 0.05, method='norm')$pred
  Real.pred.mat3[iter,2] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) + sum(test.pred.new[dis.id] == 99)/length(dis.id)*prop

  dis.diag <- which(test.pred.new == 99) ## individuals who are diagonosed as disagreed in the testing data
  Power.3[iter] <- sum(dis.diag %in% dis.id) / length(dis.diag) ## Proportion of testing data who are correctly diagonosed: Power 
}
colnames(Real.pred.mat3) <- c("Mclust","Two-stage")
	save.image(“R_Result_DryBean.RData”)

	## Case IV : Disagreement problem on the smallest class ##
niter <- 100
N.tr <- 10000
N.st <- N - 10000
test.id <- which(class == 2) # The second class was 
Real.pred.mat4 <- matrix(0,niter,2)
Power.4 <- rep(0,niter)
for (iter in 1:niter){
  set.seed(iter)
  dis.id.tmp <- which(class == 2) # The second class was disagreed
  dis.id <- dis.id.tmp[sample(1:522,300,replace = FALSE)]
  tmp <- sample((1:N)[-dis.id.tmp], N.st-length(dis.id), replace = FALSE)
  test.id <- c(tmp,dis.id);rm(tmp) # test data contains class II + more individuals
  train.dat <- dat[-test.id,]
  test.dat <- dat[test.id,]
  train.cls <- class[-test.id]
  test.cls <- class[test.id]
  dis.id <- which(test.cls == 2) # Disagreed individuals
  test.cls[dis.id] <- 99 # Disagreed memberships 
  prop <- length(dis.id) / length(test.cls) # 300/3611

  train.mod <- Mclust(train.dat, 6) # Because we know that there are now five clusters only
  test.pred <- predict(train.mod, test.dat)$classification
  Real.pred.mat4[iter,1] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) # Testing accuracy under conventional method

  test.pred.new <- Mclust.machine(train.dat, test.dat, mod=train.mod, ncls = 6, nboot = 100, alpha = 0.05, method='norm')$pred
  Real.pred.mat4[iter,2] <- Rand.index(test.pred[-dis.id], test.cls[-dis.id])*(1-prop) + sum(test.pred.new[dis.id] == 99)/length(dis.id)*prop

  dis.diag <- which(test.pred.new == 99) ## individuals who are diagonosed as disagreed in the testing data
  Power.4[iter] <- sum(dis.diag %in% dis.id) / length(dis.diag) ## Proportion of testing data who are correctly diagonosed: Power 
}
colnames(Real.pred.mat4) <- c("Mclust","Two-stage")
	save.image(“R_Result_DryBean.RData”)

	#######################################
	## End of the Dry bean data analysis ##
	#######################################

	## Print Table 9 ##

table(class)

	## Print Table 10 ##
rbind(
unname(summary(Real.pred.mat1)[,1]),
unname(summary(Real.pred.mat1)[,2]),
unname(summary(Real.pred.mat3)[,1]),
unname(summary(Real.pred.mat3)[,2]),
unname(summary(Real.pred.mat4)[,1]),
unname(summary(Real.pred.mat4)[,2]),
unname(summary(Real.pred.mat2)[,1]),
unname(summary(Real.pred.mat2)[,2])
)
